@@ -2,7 +2,7 @@
  * status.c:  the command-line's portion of the "svn status" command
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004,2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -28,6 +28,7 @@
 #include "svn_time.h"
 #include "cl.h"
 #include "svn_private_config.h"
+#include "tree-conflicts.h"
 
 /* Return the single character representation of STATUS */
 static char
@@ -53,6 +54,18 @@ generate_status_code(enum svn_wc_status_kind status)
     }
 }
 
+/* Return the single character representation of the switched column
+   status. */
+static char
+generate_switch_column_code(const svn_wc_status2_t *status)
+{
+  if (status->switched)
+    return 'S';
+  else if (status->file_external)
+    return 'X';
+  else
+    return ' ';
+}
 
 /* Return the detailed string representation of STATUS */
 static const char *
@@ -76,7 +89,6 @@ generate_status_desc(enum svn_wc_status_kind status)
     case svn_wc_status_unversioned: return "unversioned";
     default:                        abort();
     }
-  return NULL;
 }
 
 
@@ -87,9 +99,26 @@ print_status(const char *path,
              svn_boolean_t detailed,
              svn_boolean_t show_last_committed,
              svn_boolean_t repos_locks,
-             svn_wc_status2_t *status,
+             const svn_wc_status2_t *status,
              apr_pool_t *pool)
 {
+  enum svn_wc_status_kind text_status = status->text_status;
+  char tree_status_code = ' ';
+  const char *tree_desc_line = "";
+
+  /* To indicate this node is the victim of a tree conflict, we show
+     'C' in the tree-conflict column, overriding any other status.
+     We also print a separate line describing the nature of the tree
+     conflict. */
+  if (status->tree_conflict)
+    {
+      const char *desc;
+      tree_status_code = 'C';
+      svn_cl__get_human_readable_tree_conflict_description(
+        &desc, status->tree_conflict, pool);
+      tree_desc_line = apr_psprintf(pool, "\n      >   %s", desc);
+    }
+
   if (detailed)
     {
       char ood_status, lock_status;
@@ -154,53 +183,57 @@ print_status(const char *path,
 
           SVN_ERR
             (svn_cmdline_printf(pool,
-                                "%c%c%c%c%c%c %c   %6s   %6s %-12s %s\n",
-                                generate_status_code(status->text_status),
+                                "%c%c%c%c%c%c%c %c   %6s   %6s %-12s %s%s\n",
+                                generate_status_code(text_status),
                                 generate_status_code(status->prop_status),
                                 status->locked ? 'L' : ' ',
                                 status->copied ? '+' : ' ',
-                                status->switched ? 'S' : ' ',
+                                generate_switch_column_code(status),
                                 lock_status,
+                                tree_status_code,
                                 ood_status,
                                 working_rev,
                                 commit_rev,
                                 commit_author,
-                                path));
+                                path,
+                                tree_desc_line));
         }
       else
         SVN_ERR
-          (svn_cmdline_printf(pool, "%c%c%c%c%c%c %c   %6s   %s\n",
-                              generate_status_code(status->text_status),
+          (svn_cmdline_printf(pool, "%c%c%c%c%c%c%c %c   %6s   %s%s\n",
+                              generate_status_code(text_status),
                               generate_status_code(status->prop_status),
                               status->locked ? 'L' : ' ',
                               status->copied ? '+' : ' ',
-                              status->switched ? 'S' : ' ',
+                              generate_switch_column_code(status),
                               lock_status,
+                              tree_status_code,
                               ood_status,
                               working_rev,
-                              path));
+                              path,
+                              tree_desc_line));
     }
   else
     SVN_ERR
-      (svn_cmdline_printf(pool, "%c%c%c%c%c%c %s\n",
-                          generate_status_code(status->text_status),
+      (svn_cmdline_printf(pool, "%c%c%c%c%c%c%c %s%s\n",
+                          generate_status_code(text_status),
                           generate_status_code(status->prop_status),
                           status->locked ? 'L' : ' ',
                           status->copied ? '+' : ' ',
-                          status->switched ? 'S' : ' ',
+                          generate_switch_column_code(status),
                           ((status->entry && status->entry->lock_token)
                            ? 'K' : ' '),
-                          path));
+                          tree_status_code,
+                          path,
+                          tree_desc_line));
 
-  SVN_ERR(svn_cmdline_fflush(stdout));
-
-  return SVN_NO_ERROR;
+  return svn_cmdline_fflush(stdout);
 }
 
 
 svn_error_t *
 svn_cl__print_status_xml(const char *path,
-                         svn_wc_status2_t *status,
+                         const svn_wc_status2_t *status,
                          apr_pool_t *pool)
 {
   svn_stringbuf_t *sb = svn_stringbuf_create("", pool);
@@ -224,9 +257,14 @@ svn_cl__print_status_xml(const char *path,
     apr_hash_set(att_hash, "copied", APR_HASH_KEY_STRING, "true");
   if (status->switched)
     apr_hash_set(att_hash, "switched", APR_HASH_KEY_STRING, "true");
+  if (status->file_external)
+    apr_hash_set(att_hash, "file-external", APR_HASH_KEY_STRING, "true");
   if (status->entry && ! status->entry->copied)
     apr_hash_set(att_hash, "revision", APR_HASH_KEY_STRING,
                  apr_psprintf(pool, "%ld", status->entry->revision));
+  if (status->tree_conflict)
+    apr_hash_set(att_hash, "tree-conflicted", APR_HASH_KEY_STRING,
+                 "true");
   svn_xml_make_open_tag_hash(&sb, pool, svn_xml_normal, "wc-status",
                              att_hash);
 
@@ -315,7 +353,7 @@ svn_cl__print_status_xml(const char *path,
 /* Called by status-cmd.c */
 svn_error_t *
 svn_cl__print_status(const char *path,
-                     svn_wc_status2_t *status,
+                     const svn_wc_status2_t *status,
                      svn_boolean_t detailed,
                      svn_boolean_t show_last_committed,
                      svn_boolean_t skip_unrecognized,
@@ -323,7 +361,7 @@ svn_cl__print_status(const char *path,
                      apr_pool_t *pool)
 {
   if (! status
-      || (skip_unrecognized && ! status->entry)
+      || (skip_unrecognized && !(status->entry || status->tree_conflict))
       || (status->text_status == svn_wc_status_none
           && status->repos_text_status == svn_wc_status_none))
     return SVN_NO_ERROR;
